@@ -350,12 +350,13 @@ def raw_einsum(
     )
 
 
-def matmul(da, db, dims=None, out_append="2", **kwargs):
+def matmul(da, db, dims=None, *, out_append="2", **kwargs):
     """Wrap :func:`numpy.linalg.matmul`.
 
     Usage examples of all arguments is available at the
     :ref:`matmul section <linalg_tutorial/matmul>` of the linear algebra module tutorial.
     """
+    rename = False
     if dims is None:
         dims = _attempt_default_dims("matmul", da.dims, db.dims)
     if len(dims) == 3:
@@ -363,10 +364,15 @@ def matmul(da, db, dims=None, out_append="2", **kwargs):
         dims1 = [dim1, dim2]
         dims2 = [dim2, dim3]
         out_dims = [dim1, dim3]
-        if dim3 in da.dims:
-            da = da.rename({dim3: dim3 + out_append})
-        if dim1 in db.dims:
-            db = db.rename({dim1: dim1 + out_append})
+        if dim1 == dim3:
+            db = db.rename({dim3: dim3 + out_append})
+            dims2 = [dim2, dim3 + out_append]
+            out_dims = [dim1, dim3 + out_append]
+        else:
+            if dim3 in da.dims:
+                da = da.rename({dim3: dim3 + out_append})
+            if dim1 in db.dims:
+                db = db.rename({dim1: dim1 + out_append})
     elif len(dims) != 2:
         raise ValueError(
             "matmul can be one of '[str, str]', '[str, str, str]' or '[[str, str], [str, str]]'"
@@ -376,10 +382,15 @@ def matmul(da, db, dims=None, out_append="2", **kwargs):
         dims2 = dims
         out_dims = dims
     else:
-        dims1 = dims[0]
-        dims2 = dims[1]
-        out_dims = [dims[0][0], dims[1][1]]
-    return xr.apply_ufunc(
+        rename = True
+        dim11, dim12 = dims[0]
+        dim21, dim22 = dims[1]
+        da = da.rename({dim11: "__aux_dim11__", dim12: "__aux_dim12__"})
+        db = db.rename({dim21: "__aux_dim21__", dim22: "__aux_dim22__"})
+        dims1 = ["__aux_dim11__", "__aux_dim12__"]
+        dims2 = ["__aux_dim21__", "__aux_dim22__"]
+        out_dims = ["__aux_dim11__", "__aux_dim22__"]
+    matmul_aux = xr.apply_ufunc(
         np.matmul,
         da,
         db,
@@ -387,6 +398,11 @@ def matmul(da, db, dims=None, out_append="2", **kwargs):
         output_core_dims=[out_dims],
         **kwargs,
     )
+    if rename:
+        return matmul_aux.rename(
+            __aux_dim11__=dim11, __aux_dim22__=dim22 + out_append if dim22 == dim11 else dim22
+        )
+    return matmul_aux
 
 
 def matrix_transpose(da, dims):
@@ -437,7 +453,7 @@ def cholesky(da, dims=None, **kwargs):
     )
 
 
-def qr(da, dims=None, mode="reduced", out_append="2", **kwargs):  # pragma: no cover
+def qr(da, dims=None, *, mode="reduced", out_append="2", **kwargs):  # pragma: no cover
     """Wrap :func:`numpy.linalg.qr`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
@@ -473,7 +489,7 @@ def qr(da, dims=None, mode="reduced", out_append="2", **kwargs):  # pragma: no c
 
 
 def svd(
-    da, dims=None, full_matrices=True, compute_uv=True, hermitian=False, out_append="2", **kwargs
+    da, dims=None, *, full_matrices=True, compute_uv=True, hermitian=False, out_append="2", **kwargs
 ):
     """Wrap :func:`numpy.linalg.svd`.
 
@@ -522,7 +538,7 @@ def eig(da, dims=None, **kwargs):
     )
 
 
-def eigh(da, dims=None, UPLO="L", **kwargs):  # pylint: disable=invalid-name
+def eigh(da, dims=None, *, UPLO="L", **kwargs):  # pylint: disable=invalid-name
     """Wrap :func:`numpy.linalg.eigh`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
@@ -551,7 +567,7 @@ def eigvals(da, dims=None, **kwargs):
     )
 
 
-def eigvalsh(da, dims=None, UPLO="L", **kwargs):  # pylint: disable=invalid-name
+def eigvalsh(da, dims=None, *, UPLO="L", **kwargs):  # pylint: disable=invalid-name
     """Wrap :func:`numpy.linalg.eigvalsh`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
@@ -568,7 +584,7 @@ def eigvalsh(da, dims=None, UPLO="L", **kwargs):  # pylint: disable=invalid-name
     )
 
 
-def norm(da, dims=None, ord=None, **kwargs):  # pylint: disable=redefined-builtin
+def norm(da, dims=None, *, ord=None, **kwargs):  # pylint: disable=redefined-builtin
     """Wrap :func:`numpy.linalg.norm`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
@@ -587,7 +603,7 @@ def norm(da, dims=None, ord=None, **kwargs):  # pylint: disable=redefined-builti
     )
 
 
-def cond(da, dims=None, p=None, **kwargs):  # pylint: disable=invalid-name
+def cond(da, dims=None, *, p=None, **kwargs):  # pylint: disable=invalid-name
     """Wrap :func:`numpy.linalg.cond`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
@@ -607,7 +623,7 @@ def det(da, dims=None, **kwargs):
     return xr.apply_ufunc(np.linalg.det, da, input_core_dims=[dims], **kwargs)
 
 
-def matrix_rank(da, dims=None, tol=None, hermitian=False, **kwargs):
+def matrix_rank(da, dims=None, *, tol=None, hermitian=False, **kwargs):
     """Wrap :func:`numpy.linalg.matrix_rank`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
@@ -635,7 +651,7 @@ def slogdet(da, dims=None, **kwargs):
     )
 
 
-def trace(da, dims=None, offset=0, dtype=None, out=None, **kwargs):
+def trace(da, dims=None, *, offset=0, dtype=None, out=None, **kwargs):
     """Wrap :func:`numpy.trace`.
 
     Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
